@@ -126,7 +126,8 @@ let rec search_pos_type t ~pos ~env =
       List.iter tl ~f:(search_pos_type ~pos ~env);
       add_found_sig (`Type, lid.txt) ~env ~loc:t.ptyp_loc
   | Ptyp_object (fl, _) ->
-      List.iter fl ~f:(fun (_, _, ty) -> search_pos_type ty ~pos ~env)
+      List.iter fl ~f:
+        (function Oinherit ty | Otag (_, _, ty) -> search_pos_type ty ~pos ~env)
   | Ptyp_class (lid, tl) ->
       List.iter tl ~f:(search_pos_type ~pos ~env);
       add_found_sig (`Type, lid.txt) ~env ~loc:t.ptyp_loc
@@ -161,14 +162,19 @@ let rec search_pos_class_type cl ~pos ~env =
         search_pos_type ty ~pos ~env;
         search_pos_class_type cty ~pos ~env
     | Pcty_extension _ -> ()
+    | Pcty_open (_, _, cty) ->
+        search_pos_class_type cty ~pos ~env
     end
 
-let search_pos_args args ~pos ~env =
-  match args with
-    Pcstr_tuple l ->
-      List.iter l ~f:(search_pos_type ~pos ~env)
-  | Pcstr_record dl ->
-      List.iter dl ~f:(fun pld -> search_pos_type pld.pld_type ~pos ~env)
+let search_pos_arguments ~pos ~env = function
+    Pcstr_tuple l -> List.iter l ~f:(search_pos_type ~pos ~env)
+  | Pcstr_record l -> List.iter l ~f:(fun ld -> search_pos_type ld.pld_type ~pos ~env)
+
+let search_pos_constructor pcd ~pos ~env =
+  if in_loc ~pos pcd.pcd_loc then begin
+    Misc.may (search_pos_type ~pos ~env) pcd.pcd_res;
+    search_pos_arguments ~pos ~env pcd.pcd_args
+  end
 
 let search_pos_type_decl td ~pos ~env =
   if in_loc ~pos td.ptype_loc then begin
@@ -180,13 +186,7 @@ let search_pos_type_decl td ~pos ~env =
       Ptype_abstract
     | Ptype_open -> ()
     | Ptype_variant dl ->
-        List.iter dl ~f:
-          begin fun pcd ->
-            search_pos_args pcd.pcd_args ~pos ~env;
-            match pcd.pcd_res with
-              Some t -> search_pos_type t ~pos ~env
-            | None -> ()
-          end
+        List.iter dl ~f:(search_pos_constructor ~pos ~env)
     | Ptype_record dl ->
         List.iter dl ~f:(fun pld -> search_pos_type pld.pld_type ~pos ~env) in
     search_tkind td.ptype_kind;
@@ -199,7 +199,7 @@ let search_pos_type_decl td ~pos ~env =
 
 let search_pos_extension ext ~pos ~env =
   begin match ext.pext_kind with
-    Pext_decl (l, _) -> search_pos_args l ~pos ~env
+    Pext_decl (l, _) -> search_pos_arguments l ~pos ~env
   | Pext_rebind _ -> ()
   end
   
@@ -210,9 +210,9 @@ let rec search_pos_signature l ~pos ~env =
     let env = match pt.psig_desc with
       Psig_open {popen_override=ovf; popen_lid=id} ->
         let path, mt = Typetexp.find_module env Location.none id.txt in
-        begin match mt.md_type with
-          Mty_signature sign -> open_signature ovf path sign env
-        | _ -> env
+        begin match open_signature ovf path env with
+          Some env -> env
+        | None -> env
         end
     | sign_item ->
         try add_signature (Typemod.transl_signature env [pt]).sig_type env
@@ -350,7 +350,10 @@ let dummy_item =
 let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
   let env =
     match path with None -> env
-    | Some path -> Env.open_signature Fresh path sign env in
+    | Some path ->
+        match Env.open_signature Fresh path env with None -> env
+        | Some env -> env
+  in
   let title =
     match title, path with Some title, _ -> title
     | None, Some path -> string_of_path path
@@ -501,7 +504,7 @@ and view_type_decl path ~env =
       [Sig_type(ident_of_path path ~default:"t", td, Trec_first)]
 
 and view_type_id li ~env =
-  let path, decl = lookup_type li env in
+  let path = lookup_type li env in
   view_type_decl path ~env
 
 and view_class_id li ~env =
@@ -544,7 +547,7 @@ and view_decl lid ~kind ~env =
 and view_decl_menu lid ~kind ~env =
   let path, kname =
     try match kind with
-      `Type -> fst (lookup_type lid env), "Type"
+      `Type -> lookup_type lid env, "Type"
     | `Class -> fst (lookup_class lid env), "Class"
     | `Module -> lookup_module ~load:true lid env, "Module"
     | `Modtype -> fst (lookup_modtype lid env), "Module type"
@@ -704,7 +707,7 @@ let rec search_pos_structure ~pos str =
   | Tstr_recmodule bindings ->
       List.iter bindings ~f:(fun mb -> search_pos_module_expr mb.mb_expr ~pos)
   | Tstr_class l ->
-      List.iter l ~f:(fun (cl, _) -> search_pos_class_expr cl.ci_expr ~pos)
+      List.iter l ~f:(fun (ci, _) -> search_pos_class_expr ci.ci_expr ~pos)
   | Tstr_include {incl_mod=m} -> search_pos_module_expr m ~pos
   | Tstr_primitive _
   | Tstr_type _
@@ -755,6 +758,7 @@ and search_pos_class_expr ~pos cl =
           end;
         List.iter iel ~f:(fun (_,_, exp) -> search_pos_expr exp ~pos);
         search_pos_class_expr cl ~pos
+    | Tcl_open (_, _, _, _, cl)
     | Tcl_constraint (cl, _, _, _, _) ->
         search_pos_class_expr cl ~pos
     end;
@@ -786,7 +790,7 @@ and search_pos_expr ~pos exp =
         search_pos_expr exp' ~pos
       end;
       search_pos_expr exp ~pos
-  | Texp_function (_, l, _) ->
+  | Texp_function {cases=l; _} ->
       List.iter l ~f:(search_case ~pos)
   | Texp_apply (exp, l) ->
       List.iter l ~f:(fun (_, x) -> Misc.may (search_pos_expr ~pos) x);
@@ -801,8 +805,9 @@ and search_pos_expr ~pos exp =
   | Texp_construct (_, _, l) -> List.iter l ~f:(search_pos_expr ~pos)
   | Texp_variant (_, None) -> ()
   | Texp_variant (_, Some exp) -> search_pos_expr exp ~pos
-  | Texp_record (l, opt) ->
-      List.iter l ~f:(fun (_, _, exp) -> search_pos_expr exp ~pos);
+  | Texp_record {fields=l; extended_expression=opt} ->
+      Array.iter l ~f:
+        (function (_,Overridden(_,exp)) -> search_pos_expr exp ~pos | _ -> ());
       (match opt with None -> () | Some exp -> search_pos_expr exp ~pos)
   | Texp_field (exp, _, _) -> search_pos_expr exp ~pos
   | Texp_setfield (a, _, _, b) ->
@@ -847,6 +852,8 @@ and search_pos_expr ~pos exp =
       ()
   | Texp_extension_constructor _ ->
       ()
+  | Texp_letexception (_, exp) ->
+      search_pos_expr exp ~pos
   end;
   add_found_str (`Exp(`Expr, exp.exp_type)) ~env:exp.exp_env ~loc:exp.exp_loc
   end
