@@ -27,15 +27,76 @@
 
 /* Yell if a caml callback raised an exception */
 #define CAML_EXN_LOG(name) g_critical("%s: callback raised an exception", name)
-#define CAML_EXN_LOG_VERBOSE(name,exn) g_critical("%s: callback raised exception %s", name, format_caml_exception(Extract_exception(exn)))
+#define CAML_EXN_LOG_VERBOSE(name,exn) g_critical("%s: callback raised exception %s", name, caml_format_exception(Extract_exception(exn)))
 
 #include <caml/misc.h>
 #include <caml/mlvalues.h>
 #include <caml/fail.h>
 #include <caml/custom.h>
-#include <caml/version.h>
-#if OCAML_VERSION < 41000
-CAMLextern char *young_start, *young_end; /* from minor_gc.h */
+#include <caml/minor_gc.h>
+#define Is_young_block(v) \
+  (Is_block(v) && (value*)(v) < caml_young_end && (value*)(v) > caml_young_start)
+
+#ifndef Bytes_val
+#define Bytes_val String_val
+#endif
+
+/* We are still using the compatibility layer */
+#ifdef CAML_NAME_SPACE
+/* **** alloc.c */
+#define alloc caml_alloc /*SP*/
+#define alloc_small caml_alloc_small
+#define alloc_tuple caml_alloc_tuple
+#define alloc_string caml_alloc_string
+#define alloc_final caml_alloc_final
+#define copy_string caml_copy_string
+#define alloc_array caml_alloc_array
+#define copy_string_array caml_copy_string_array
+#define convert_flag_list caml_convert_flag_list
+/* **** callback.c */
+#define callback_depth caml_callback_depth
+#define callbackN_exn caml_callbackN_exn
+#define callback_exn caml_callback_exn
+#define callback2_exn caml_callback2_exn
+#define callback3_exn caml_callback3_exn
+#define callback caml_callback
+#define callback2 caml_callback2
+#define callback3 caml_callback3
+#define callbackN caml_callbackN
+/* **** custom.c */
+#define alloc_custom caml_alloc_custom
+#define register_custom_operations caml_register_custom_operations
+/* **** ints.c */
+#define copy_int32 caml_copy_int32
+#define copy_int64 caml_copy_int64
+#define copy_nativeint caml_copy_nativeint
+/* **** fail.c */
+#define external_raise caml_external_raise
+#define mlraise caml_raise /*SP*/
+#define raise_constant caml_raise_constant
+#define raise_with_arg caml_raise_with_arg
+#define raise_with_string caml_raise_with_string
+#define failwith caml_failwith
+#define invalid_argument caml_invalid_argument
+#define array_bound_error caml_array_bound_error /*SP*/
+#define raise_out_of_memory caml_raise_out_of_memory
+#define raise_stack_overflow caml_raise_stack_overflow
+#define raise_sys_error caml_raise_sys_error
+#define raise_end_of_file caml_raise_end_of_file
+#define raise_zero_divide caml_raise_zero_divide
+#define raise_not_found caml_raise_not_found
+#define raise_sys_blocked_io caml_raise_sys_blocked_io
+/* **** floats.c */
+#define copy_double caml_copy_double
+/* **** memory.c */
+#define alloc_shr caml_alloc_shr
+#define initialize caml_initialize
+#define modify caml_modify
+#define stat_alloc caml_stat_alloc
+#define stat_free caml_stat_free
+#define stat_resize caml_stat_resize
+/* **** str.c */
+#define string_length caml_string_length
 #endif
 
 CAMLexport value copy_memblock_indirected (void *src, asize_t size);
@@ -57,32 +118,35 @@ char ** strv_of_string_list (value list);
 CAMLprim value *ml_global_root_new (value v);
 CAMLexport void ml_global_root_destroy (void *data);
 
+/* Pointer conversions */
+#define Pointer_val(val) ((void*)Field(val,1))
+#define Store_pointer(val,p) (Field(val,1)=Val_bp(p))
+#define MLPointer_val(val) \
+        ((int)Field(val,1) == 2 ? &Field(val,2) : (void*)Field(val,1))
+
+#define Val_addr(ptr) (1+(value)ptr)
+#define Addr_val(val) ((void*)(val-1))
+
 /* enums <-> polymorphic variants */
 typedef struct { value key; int data; } lookup_info;
+#define Val_lookup_info(v) Val_pointer((void*)v)
+#define Lookup_info_val(v) ((const lookup_info*)Pointer_val(v))
 CAMLexport value ml_lookup_from_c (const lookup_info table[], int data);
 CAMLexport int ml_lookup_to_c (const lookup_info table[], value key);
 CAMLexport value ml_lookup_flags_getter (const lookup_info table[], int data);
 
 /* Compatibility */
-#include <gtk/gtkversion.h>
+/* #include <gtk/gtk.h>
 #if GTK_CHECK_VERSION(2,2,0) && !defined(DISABLE_GTK22)
 #define HASGTK22
 #endif
-#if GTK_CHECK_VERSION(2,4,0) && !defined(DISABLE_GTK24)
+*/
+#define HASGTK22
 #define HASGTK24
-#endif
-#if GTK_CHECK_VERSION(2,5,3) && !defined(DISABLE_GTK26)
 #define HASGTK26
-#endif
-#if GTK_CHECK_VERSION(2,8,0) && !defined(DISABLE_GTK28)
 #define HASGTK28
-#endif
-#if GTK_CHECK_VERSION(2,10,0) && !defined(DISABLE_GTK210)
 #define HASGTK210
-#endif
-#if GTK_CHECK_VERSION(2,12,0) && !defined(DISABLE_GTK212)
 #define HASGTK212
-#endif
 
 /* Wrapper generators */
 
@@ -253,7 +317,7 @@ CAMLprim value cname##_bc (value *argv, int argn) \
 	       argv[7],argv[8],argv[9],argv[10],argv[11],argv[12]); }
 
 /* result conversion */
-#define Unit(x) ((x), Val_unit)
+#define Unit(x) (((void)x), Val_unit)
 #define Id(x) x
 #define Val_char Val_int
 
@@ -283,7 +347,7 @@ CAMLprim value cname##_bc (value *argv, int argn) \
  else if (l <= Max_young_wosize) { int i; ret = alloc_tuple(l); \
    for(i=0;i<l;i++) Field(ret,i) = conv(src[i]); } \
  else { int i; ret = alloc_shr(l,0); \
-   for(i=0;i<l;i++) initialize (&Field(ret,i), conv(src[i])); }
+   for(i=0;i<l;i++) caml_initialize (&Field(ret,i), conv(src[i])); }
 
 #define Make_Val_final_pointer(type, init, final, adv) \
 static void ml_final_##type (value val) \
@@ -294,7 +358,7 @@ static struct custom_operations ml_custom_##type = \
 CAMLprim value Val_##type (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
   ret = ml_alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
-  initialize (&Field(ret,1), (value) p); init(p); return ret; }
+  caml_initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Make_Val_final_pointer_ext(type, ext, init, final, adv) \
 static void ml_final_##type##ext (value val) \
@@ -305,7 +369,7 @@ static struct custom_operations ml_custom_##type##ext = \
 CAMLprim value Val_##type##ext (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
   ret = ml_alloc_custom (&ml_custom_##type##ext, sizeof(value), adv, 1000); \
-  initialize (&Field(ret,1), (value) p); init(p); return ret; }
+  caml_initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Make_Val_final_pointer_compare(type, init, comp, final, adv) \
 static void ml_final_##type (value val) \
@@ -318,15 +382,7 @@ static struct custom_operations ml_custom_##type = \
 CAMLprim value Val_##type (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
   ret = ml_alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
-  initialize (&Field(ret,1), (value) p); init(p); return ret; }
-
-#define Pointer_val(val) ((void*)Field(val,1))
-#define Store_pointer(val,p) (Field(val,1)=Val_bp(p))
-#define MLPointer_val(val) \
-        ((int)Field(val,1) == 2 ? &Field(val,2) : (void*)Field(val,1))
-
-#define Val_addr(ptr) (1+(value)ptr)
-#define Addr_val(val) ((void*)(val-1))
+  caml_initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Wosize_asize(x) ((x-1)/sizeof(value)+1)
 #define Wosizeof(x) Wosize_asize(sizeof(x))
